import {Point, BackgroundPaniter, ViewTool} from './UIUtils';

/**
 * 编辑器基类
 * @author zhaoyk
 */
export abstract class Editor {
	
	protected container: any;
	
	protected canvas: HTMLCanvasElement | undefined;
	
	protected ctx: CanvasRenderingContext2D | undefined;
	
	protected canvasSize: Point;
	
	protected initialised = false;
	
	protected editorPart: any; //vue编辑界面组件
	
	protected bgPainter: BackgroundPaniter;

	protected state = 0;

	protected selection: Selection;

	margin = 20;
	
	constructor(container: any) {
		this.container = container;
	}
	
	setEditorPart(editorPart: any): void {
		this.editorPart = editorPart;
	}
	
	show(data: any) : void {
		if(!this.initialised)
			this.initPanel();
			
		this.showData(data);
	}
	
	private initPanel(): void{
		this.canvasSize = this.initCanvasSize();

		const canvas = document.createElement('canvas');		
		canvas.width = this.canvasSize.x; // 实际渲染像素
		canvas.height = this.canvasSize.y; // 实际渲染像素
		canvas.style.width = this.canvasSize.x + 'px'; // 控制显示大小
		canvas.style.height = (this.canvasSize.y - 4) + 'px'; // 控制显示大小
		canvas.style.position = 'relative';
								
		this.container.appendChild(canvas);
		this.canvas = canvas;
		
		const ctx = canvas.getContext('2d');
		if (ctx) {
			ctx.translate(0.5, 0.5);
			this.ctx = ctx;
		} else {
			throw new Error('Canvas 2D context not found, please check it is running in Browser environment.');
		}
				
		this.bgPainter = new BackgroundPaniter(canvas, ctx);		
			
		this.addListeners();

		this.initialised = true;
	}
	
	protected abstract initCanvasSize(): Point;
	
	abstract addListeners(): void;

	abstract showData(data: any): void;

	abstract redraw(): void;
	
	protected beforeRedraw(): void {
		this.ctx.clearRect(-1, -1, this.canvas.width + 1, this.canvas.height + 1);
		this.bgPainter.paint();
	}
	
}

export class Selection {
	
	obj: any;
	
	objType: string;
	
	setSel(obj: any, objType?: string): void{
		this.obj = obj;
		this.objType = objType;
	}
	
	getSel() : any {
		return this.obj;
	}
	
	getSelType(): string{
		return this.objType;
	}
	
}